#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File: chanconfig/multiple_config.py
# Author: Jimin Huang <huangjimin@whu.edu.cn>
# Date: 18.05.2018
from __future__ import absolute_import
import logging
import pkg_resources
import yaml

from yaml.parser import ParserError

from chanconfig import utils
from .config import ReadConfigurationError


class MultipleConfig(object):
    """Class for configurations from stdin, config files and env.
    """

    def __init__(
        self, path, package=None, arguments={}, argument_settings={}
    ):
        """Initialize class with given path and optional package name.

        Args:
            path: a str
            package: a str, default None
            arguments: (dict) command arguments
            argument_settings: (dict) as {
                key: (env_name, process_method, default)
            }
        """
        self.logger = logging.getLogger(__name__)

        if package:
            try:
                path = pkg_resources.resource_filename(package, path)
            except ImportError as error:
                self.logger.exception(error)
                raise ReadConfigurationError(path, str(error))

        try:
            with open(path) as ymlfile:
                self.dict = yaml.load(ymlfile)
        except (IOError, ParserError) as error:
            self.logger.exception(error)
            raise ReadConfigurationError(path, str(error))

        self.update(self.process_arguments(arguments))
        self.update(self.process_arguments(argument_settings))

        self.logger.info(
            'Successfully read configurations from {0}'.format(path)
        )

    def __getattr__(self, name):
        return getattr(self.dict, name)

    def __eq__(self, other):
        return self.dict == other

    def __repr__(self):
        return str(self.dict)

    def update(self, update_dict):
        """Update values from another dict

        Args:
            update_dict: a ``dict``
        """
        self.dict = utils.deep_update(self.dict, update_dict)

    def process_arguments(self, arguments):
        """Process argument settings and arguments to a dict

        Args:
            arguments: (dict) settings of specified keys

        Return:
            (dict) as {key: val}
        """
        new_val_dict = {}
        update_value = None
        for key, val in arguments.items():
            key = key.lstrip('--')
            keys = key.split('.')

            try:
                existing_value = utils.deep_find(self.dict, keys)
            except KeyError:
                existing_value = None

            try:
                update_value = utils.generate_update_value(val, existing_value)
            except AssertionError:
                update_value = val
            except Exception as e:
                self.logger.exception(e)
                self.logger.info('Set default value of {}'.format(key))
                update_value = val[2]

            new_val_dict = utils.update_dict_by_path(
                new_val_dict, keys, update_value
            )
        return new_val_dict



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File: chanconfig/utils.py
# Author: Cai Mingshi <i@unoiou.com>
# Date: 20.08.2018
from __future__ import absolute_import
import os
from collections import Mapping


def deep_update(orig_dict, update_dict):
    """Deeply update nested dict.

    Args:
        orig_dict: a `dict`
        update_dict: a `dict`

    Return:
        a `dict`
    """
    for key, val in update_dict.items():
        temp = val
        if isinstance(val, Mapping):
            temp = deep_update(orig_dict.get(key, {}), val)
        orig_dict[key] = temp
    return orig_dict


def update_dict_by_path(dict_obj, path, value):
    """Update dict with value py path.

    Args:
        dict_obj: a `dict`
        path: a `list`
        value: an `object`

    Return:
        a `dict`
    """
    obj = dict_obj
    for index, val in enumerate(path):
        if index + 1 == len(path):
            obj[val] = value
        obj.setdefault(val, {})
        obj = obj[val]
    return dict_obj


def deep_find(dict_obj, path):
    """Deeply find element in dict by path.

    Args:
        dict_obj: a `dict`
        path: a `list`

    Return:
        an 'object'

    Raises:
        `KeyError`
    """
    obj = dict_obj
    for key in path:
        obj = obj[key]
    return obj


def generate_update_value(val, existing_value):
    """Generate update value with priority.

    Args:
        val: (obj) setting value of specified key
        existing_value: (obj) existing setting value of specified key

    Return:
        (obj)

    Raises:
        `AssertionError`
        `ValueError`
    """
    assert isinstance(val, tuple)

    update_value = None
    env, process, default = val

    update_value = (env and os.getenv(env)) or existing_value or default

    if process is not None:
        update_value = process(update_value)
    return update_value



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File: chanconfig/__init__.py
# Author: Jimin Huang <huangjimin@whu.edu.cn>
# Date: 19.10.2017
from __future__ import absolute_import
from .config import Config, ReadConfigurationError
from .multiple_config import MultipleConfig


__all__ = ['Config', 'MultipleConfig', 'ReadConfigurationError']
__version__ = '0.0.9'
